package vfs // import "github.com/docker/docker/daemon/graphdriver/vfs"

import (
	"github.com/docker/docker/daemon/graphdriver"
	"github.com/docker/docker/daemon/graphdriver/quota"
	"github.com/sirupsen/logrus"
)

type driverQuota struct {
	quotaCtl *quota.Control
}

func setupDriverQuota(driver *Driver) {
	// Probe fs type before setting quota, now only supports xfs and extfs
	fsMagic, err := graphdriver.GetFSMagic(driver.home)
	if err != nil {
		return
	}
	fsName, ok := graphdriver.FsNames[fsMagic]
	if !ok {
		return
	}

	if quotaCtl, err := quota.NewControl(driver.home, fsName); err == nil {
		driver.quotaCtl = quotaCtl
	} else if err != quota.ErrQuotaNotSupported {
		logrus.Warnf("Unable to setup quota: %v\n", err)
	}
}

func (d *Driver) setupQuota(dir string, size uint64) error {
	return d.quotaCtl.SetQuota(dir, quota.Quota{Size: size})
}

func (d *Driver) quotaSupported() bool {
	return d.quotaCtl != nil
}
