// +build linux

package fs

import (
	"fmt"
	"strconv"

	"github.com/opencontainers/runc/libcontainer/cgroups"
	"github.com/opencontainers/runc/libcontainer/configs"
	"path/filepath"
)

type FilesGroup struct {
}

func (s *FilesGroup) Name() string {
	return "files"
}

func (s *FilesGroup) Apply(d *cgroupData) error {
	_, err := d.join("files")
	if err != nil && !cgroups.IsNotFound(err) {
		return err
	}
	return nil
}

func (s *FilesGroup) Set(path string, cgroup *configs.Cgroup) error {
	if cgroup.Resources.FilesLimit != 0 {
		// "max" is the fallback value.
		limit := "max"
		if cgroup.Resources.FilesLimit > 0 {
			limit = strconv.FormatInt(cgroup.Resources.FilesLimit, 10)
		}

		if err := writeFile(path, "files.limit", limit); err != nil {
			return err
		}
	}

	return nil
}

func (s *FilesGroup) Remove(d *cgroupData) error {
	return removePath(d.path("files"))
}

func (s *FilesGroup) GetStats(path string, stats *cgroups.Stats) error {
	usage, err := getCgroupParamUint(path, "files.usage")
	if err != nil {
		return fmt.Errorf("failed to parse files.usage - %s", err)
	}

	maxString, err := getCgroupParamString(path, "files.limit")
	if err != nil {
		return fmt.Errorf("failed to parse files.limit - %s", err)
	}

	// Default if files.limit == "max" is 0 -- which represents "no limit".
	var max uint64
	if maxString != "max" {
		max, err = parseUint(maxString, 10, 64)
		if err != nil {
			return fmt.Errorf("failed to parse files.limit -- unable to parse %q as a uint from Cgroup file %q", maxString, filepath.Join(path, "file.limits"))
		}
	}

	stats.FilesStats.Usage = usage
	stats.FilesStats.Limit = max
	return nil
}
